<?php

namespace Espo\Modules\RealEstate\Hooks\RealEstateProperty;

class EmailRequester extends \Espo\Core\Hooks\Base
{
    public static $order = 16;

    protected function init()
    {
        $this->addDependency('serviceFactory');
    }

    protected function getServiceFactory()
    {
        return $this->getInjection('serviceFactory');
    }

    public function afterSave($entity)
    {
        if (!$this->getConfig()->get('realEstateEmailSending')) {
            return;
        }

        $scheduleEmailSending = $entity->isNew();
        $checkFields = [
            "type",
            "propertyType",
            "locations",
            "price",
            "square",
            "bedroomCount",
            "yearBuilt",
            "bathroomCount",
            "floor",
            "floorCount"
        ];

        if (!$scheduleEmailSending) {
            foreach ($checkFields as $field) {
                if ($entity->isFieldChanged($field)) {
                    $scheduleEmailSending = true;
                    break;
                }
            }
        }

        if ($scheduleEmailSending) {
            $pdo = $this->getEntityManager()->getPDO();

            $service = $this->getServiceFactory()->create($entity->getEntityName());
            $matchingRequests = $service->findLinkedEntitiesMatchingRequests($entity->id, array());

            foreach ($matchingRequests['collection'] as $request) {
                $sql = "
                    SELECT COUNT(id) as cnt FROM job
                    WHERE method = 'sendMatchesEmail' AND
                        service_name = 'RealEstateSendMatches' AND
                        deleted = 0 AND
                        status = 'Pending' AND
                        data LIKE '%{$entity->id}%' AND
                        data LIKE '%{$request->id}%'
                ";

                $sth = $pdo->prepare($sql);
                $sth->execute();

                $res = $sth->fetch(\PDO::FETCH_ASSOC);

                if (!$res['cnt']) {
                    $job = $this->getEntityManager()->getEntity('Job');
                    $job->set( array(
                            'method' => 'sendMatchesEmail',
                            'serviceName' => 'RealEstateSendMatches',
                            'executeTime' => date('Y-m-d H:i:s'),
                            'data' => json_encode(array(
                                "requestId" => $request->id,
                                "propertyId" => $entity->id,
                                "isUpdated" => !$entity->isNew(),
                            ))
                        )
                    );
                    $this->getEntityManager()->saveEntity($job);
                }
            }
        }
    }
}
